use Test2::V0;

use Sub::Meta;
use Sub::Meta::Param;
use Sub::Meta::Parameters;
use Sub::Meta::Returns;
use Sub::Meta::Library;
use Sub::Meta::Creator;
use Sub::Meta::Finder::Default;
use Sub::Meta::CreatorFunction;
use Sub::Meta::Test;
use Sub::Meta::Type;
use Sub::Meta::TypeSub;
use Types::Sub;

pass;

done_testing;

